@@ -814,8 +814,8 @@ test_compressed_content_encoding (void)
 
 	/* Requires:
 	 * http://git.gnome.org/browse/gvfs/commit/?id=6929e9f9661b4d1e68f8912d8e60107366255a47
-	 * http://thread.gmane.org/gmane.comp.gnome.apps.rhythmbox.devel/11887 */
-	result = simple_parser_test ("http://escapepod.org/podcast.xml");
+	 * https://mail.gnome.org/archives/rhythmbox-devel/2011-November/thread.html#00010 */
+	result = simple_parser_test ("https://escapepod.org/feed/");
 	g_assert_cmpint (result, ==, TOTEM_PL_PARSER_RESULT_SUCCESS);
 }
 
@@ -1011,6 +1011,14 @@ test_parsing_not_really_php (void)
 	g_free (uri);
 }
 
+static void
+test_parsing_remote_mp3 (void)
+{
+	g_test_bug ("19");
+	/* URL from https://gitlab.gnome.org/GNOME/totem-pl-parser/issues/19 */
+	g_assert_cmpint (simple_parser_test ("http://feeds.soundcloud.com/stream/303432626-opensourcesecuritypodcast-episode-28-rsa-conference-2017.mp3"), ==, TOTEM_PL_PARSER_RESULT_UNHANDLED);
+}
+
 static void
 test_parsing_not_really_php_but_html_instead (void)
 {
@@ -1089,6 +1097,97 @@ test_async_parsing_signal_order (void)
 	g_main_loop_unref (data.mainloop);
 }
 
+static void
+add_pl_iter_metadata (TotemPlPlaylist     *playlist,
+		      TotemPlPlaylistIter *pl_iter)
+{
+	totem_pl_playlist_set (playlist, pl_iter,
+			       TOTEM_PL_PARSER_FIELD_URI, "file:///fake/uri.mp4",
+			       TOTEM_PL_PARSER_FIELD_TITLE, "custom title",
+			       TOTEM_PL_PARSER_FIELD_SUBTITLE_URI, "file:///fake/uri.srt",
+			       TOTEM_PL_PARSER_FIELD_PLAYING, "true",
+			       TOTEM_PL_PARSER_FIELD_CONTENT_TYPE, "video/mp4",
+			       TOTEM_PL_PARSER_FIELD_STARTTIME, "1",
+			       NULL);
+}
+
+static void
+test_saving_sync (void)
+{
+	g_autoptr(TotemPlParser) parser = NULL;
+	g_autoptr(TotemPlPlaylist) playlist = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GFile) output = NULL;
+	TotemPlPlaylistIter pl_iter;
+	g_autofree char *path = NULL;
+	int fd;
+
+	parser = totem_pl_parser_new ();
+	playlist = totem_pl_playlist_new ();
+	fd = g_file_open_tmp (NULL, &path, &error);
+	g_assert_no_error (error);
+	close (fd);
+	output = g_file_new_for_path (path);
+
+	totem_pl_parser_save (parser,
+			      playlist,
+			      output,
+			      NULL, TOTEM_PL_PARSER_XSPF, &error);
+	g_assert_error (error, TOTEM_PL_PARSER_ERROR, TOTEM_PL_PARSER_ERROR_EMPTY_PLAYLIST);
+	g_clear_error (&error);
+
+	totem_pl_playlist_append (playlist, &pl_iter);
+	add_pl_iter_metadata (playlist, &pl_iter);
+	totem_pl_parser_save (parser,
+			      playlist,
+			      output,
+			      NULL, TOTEM_PL_PARSER_XSPF, &error);
+	g_assert_no_error (error);
+}
+
+static void
+test_saving_sync_cb (GObject       *source_object,
+		     GAsyncResult  *res,
+		     gpointer       user_data)
+{
+	GMainLoop *loop = user_data;
+	g_autoptr(GError) error = NULL;
+
+	totem_pl_parser_save_finish (TOTEM_PL_PARSER (source_object), res, &error);
+	g_assert_no_error (error);
+	g_main_loop_quit (loop);
+}
+
+static void
+test_saving_async (void)
+{
+	g_autoptr(TotemPlParser) parser = NULL;
+	g_autoptr(TotemPlPlaylist) playlist = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GFile) output = NULL;
+	TotemPlPlaylistIter pl_iter;
+	g_autofree char *path = NULL;
+	GMainLoop *loop;
+	int fd;
+
+	parser = totem_pl_parser_new ();
+	playlist = totem_pl_playlist_new ();
+	fd = g_file_open_tmp (NULL, &path, &error);
+	g_assert_no_error (error);
+	close (fd);
+	output = g_file_new_for_path (path);
+
+	totem_pl_playlist_append (playlist, &pl_iter);
+	add_pl_iter_metadata (playlist, &pl_iter);
+	loop = g_main_loop_new (NULL, FALSE);
+	totem_pl_parser_save_async (parser,
+				    playlist,
+				    output,
+				    NULL, TOTEM_PL_PARSER_XSPF,
+				    NULL, test_saving_sync_cb, loop);
+	g_main_loop_run (loop);
+}
+
 #define MAX_DESCRIPTION_LEN 128
 #define DATE_BUFSIZE 512
 #define PRINT_DATE_FORMAT "%Y-%m-%dT%H:%M:%SZ"
@@ -1359,6 +1458,9 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/parsing/dir_recurse", test_directory_recurse);
 		g_test_add_func ("/parser/parsing/async_signal_order", test_async_parsing_signal_order);
 		g_test_add_func ("/parser/parsing/wma_asf", test_parsing_wma_asf);
+		g_test_add_func ("/parser/parsing/remote_mp3", test_parsing_remote_mp3);
+		g_test_add_func ("/parser/saving/sync", test_saving_sync);
+		g_test_add_func ("/parser/saving/async", test_saving_async);
 
 		return g_test_run ();
 	}
