@@ -415,13 +415,12 @@ cd_cache_new (const char *dev,
   found = cd_cache_get_dev_from_volumes (mon, device, &mountpoint, &volume);
   if (!found) {
     g_set_error (error, TOTEM_PL_PARSER_ERROR, TOTEM_PL_PARSER_ERROR_NO_DISC,
-	_("No media in drive for device '%s'."),
+	_("No media in drive for device “%s”."),
 	device);
     g_free (device);
     return NULL;
   }
 
-  /* create struture */
   cache = g_new0 (CdCache, 1);
   cache->device = device;
   cache->mountpoint = mountpoint;
